import textwrap
from typing import Callable, Optional, List

import click
from pycarlo.core import Client, Query
from tabulate import tabulate

from montecarlodata.errors import manage_errors


class MonitorService:
    MONITORS_HEADERS = ('Monitor UUID', 'Type', 'Namespace', 'Description', 'Last Update Time')
    MORE_MONITOR_MESSAGE = 'There are more monitors available. Increase the limit to view them.'
    PSEUDO_MONITOR_TYPE_CB_COMPATIBLE = 'CIRCUIT_BREAKER_COMPATIBLE'
    MONITOR_TYPE_STATS = 'STATS'
    MONITOR_TYPE_CATEGORIES = 'CATEGORIES'
    MONITOR_TYPE_JSON_SCHEMA = 'JSON_SCHEMA'
    MONITOR_TYPE_CUSTOM_SQL = 'CUSTOM_SQL'
    MONITOR_TYPE_TABLE_METRIC = 'TABLE_METRIC'  # Legacy Volume SLIs
    MONITOR_TYPE_FRESHNESS = 'FRESHNESS'
    MONITOR_TYPE_VOLUME = 'VOLUME'
    MONITOR_TYPES = [
        PSEUDO_MONITOR_TYPE_CB_COMPATIBLE,
        MONITOR_TYPE_CUSTOM_SQL,
        MONITOR_TYPE_TABLE_METRIC,
        MONITOR_TYPE_FRESHNESS,
        MONITOR_TYPE_VOLUME,
        MONITOR_TYPE_STATS,
        MONITOR_TYPE_CATEGORIES,
        MONITOR_TYPE_JSON_SCHEMA,
    ]

    def __init__(
        self,
        client: Optional[Client] = None,
        print_func: Optional[Callable] = click.echo,
    ):
        self._pycarlo_client = client or Client()
        self._print_func = print_func

    @manage_errors
    def list_monitors(self, limit=100, namespaces: List[str] = None, monitor_types: List[str] = None):
        """
        Get all monitors filter by namespaces and monitor_types
        """
        kwargs = {
            "limit": limit + 1
        }
        if namespaces:
            kwargs["namespaces"] = namespaces
        if monitor_types:
            kwargs["monitor_types"] = monitor_types
        query = Query()
        query.get_monitors(**kwargs).__fields__(
            'uuid',
            'monitor_type',
            'namespace',
            'description',
            'last_update_time',
        )
        response = self._pycarlo_client(query)
        monitors = [(item.uuid, item.monitor_type, item.namespace,
                     textwrap.fill(item.description, width=70) if item.description else "", item.last_update_time)
                    for item in response.get_monitors]
        more_ns_available = False
        if len(monitors) > limit:
            monitors = monitors[:-1]
            more_ns_available = True
        self._print_func(tabulate(monitors, headers=self.MONITORS_HEADERS, tablefmt='fancy_grid'))

        if more_ns_available:
            self._print_func(self.MORE_MONITOR_MESSAGE)



# Query response fields
MATCH_AND_CREATE_BI_WH_SRC_GQL_OPERATION = 'matchAndCreateBiWarehouseSources'
MATCH_AND_CREATE_BI_WH_SRC_GQL_RESPONSE_FIELD = 'match_and_create_bi_warehouse_sources'
MATCHING_BI_WH_SRC_GQL_RESPONSE_FIELD = 'matching_bi_warehouse_sources'
MATCH_SUCCESSFUL_GQL_RESPONSE_FIELD = 'match_successful'



from unittest import TestCase
from unittest.mock import Mock, patch, call

import click
from box import BoxList, Box

from montecarlodata.common.data import MonolithResponse
from montecarlodata.insights.data_insights import InsightsService
from montecarlodata.queries.insights import GET_INSIGHTS, GET_INSIGHT_REPORT
from montecarlodata.utils import GqlWrapper
from tests.test_common_user import _SAMPLE_CONFIG

SAMPLE_RAW = 'loki'
SAMPLE_INSIGHT = 'key_assets'
SAMPLE_INSIGHT_URL = 'https://montecarlodata.com'
SAMPLE_S3_DESTINATION = 's3://bucket/prefix/object'
SAMPLE_FS_DESTINATION = 'file://folder1/folder2/object'


class MockRequest:
    raw = SAMPLE_RAW


class InsightServiceTest(TestCase):
    def setUp(self) -> None:
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._service = InsightsService(
            _SAMPLE_CONFIG,
            request_wrapper=self._request_wrapper_mock
        )

    @patch('montecarlodata.insights.data_insights.click')
    def test_echo_insights(self, click_mock):
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data=BoxList([{'description': 'description', 'title': 'Key Assets', 'available': True}]),
            errors=None
        )

        self._service.echo_insights(table_format='plain')
        self._request_wrapper_mock.make_request_v2.assert_called_once_with(query=GET_INSIGHTS, operation='getInsights')
        click_mock.echo.assert_called_once_with('Insight     Description    Available\nKey Assets  description.   True')

    def test_get_insight_with_bad_scheme(self):
        with self.assertRaises(click.exceptions.Abort):
            self._service.get_insight(insight=SAMPLE_INSIGHT, destination='foo')

        with self.assertRaises(click.exceptions.Abort):
            self._service.get_insight(insight=SAMPLE_INSIGHT, destination='http://foo')

    @patch.object(InsightsService, '_save_insight_to_disk')
    @patch.object(InsightsService, '_save_insight_to_s3')
    def test_get_insight_routing_s3(self, save_to_s3_mock, save_to_disk_mock):
        self._test_get_insight_routing(destination=SAMPLE_S3_DESTINATION)

        save_to_disk_mock.assert_not_called()
        save_to_s3_mock.assert_called_once_with(
            insight_url=SAMPLE_INSIGHT_URL,
            destination='bucket/prefix/object',
            aws_profile=None
        )

    @patch.object(InsightsService, '_save_insight_to_disk')
    @patch.object(InsightsService, '_save_insight_to_s3')
    def test_get_insight_routing_fs(self, save_to_s3_mock, save_to_disk_mock):
        self._test_get_insight_routing(destination=SAMPLE_FS_DESTINATION)

        save_to_disk_mock.assert_called_once_with(
            insight_url=SAMPLE_INSIGHT_URL,
            destination='folder1/folder2/object',
            aws_profile=None
        )
        save_to_s3_mock.assert_not_called()

    @patch('montecarlodata.insights.data_insights.click')
    @patch.object(InsightsService, '_get_insight_url')
    @patch.object(InsightsService, '_save_insight_to_disk')
    @patch.object(InsightsService, '_save_insight_to_s3')
    def test_get_insight_dry(self, save_to_s3_mock, save_to_disk_mock, get_mock, mock_click):
        get_mock.return_value = SAMPLE_INSIGHT_URL

        service = InsightsService(_SAMPLE_CONFIG, request_wrapper=self._request_wrapper_mock)
        service.get_insight(insight=SAMPLE_INSIGHT, destination=SAMPLE_FS_DESTINATION, dry=True)
        get_mock.assert_called_once_with(insight=SAMPLE_INSIGHT)
        mock_click.echo.assert_called_once_with(SAMPLE_INSIGHT_URL)

        save_to_disk_mock.assert_not_called()
        save_to_s3_mock.assert_not_called()

    @patch('montecarlodata.insights.data_insights.click')
    @patch.object(InsightsService, '_get_insight_url')
    def _test_get_insight_routing(self, get_mock, click_mock, destination: str):
        get_mock.return_value = SAMPLE_INSIGHT_URL

        service = InsightsService(_SAMPLE_CONFIG, request_wrapper=self._request_wrapper_mock)
        service.get_insight(insight=SAMPLE_INSIGHT, destination=destination)
        get_mock.assert_called_once_with(insight=SAMPLE_INSIGHT)
        click_mock.echo.assert_has_calls(
            [
                call(f"Saving insight to '{destination}'."),
                call('Complete. Have a nice day!')
            ]
        )
        self.assertEqual(click_mock.echo.call_count, 2)

    def test_get_insights_url(self):
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(data=Box({'url': SAMPLE_INSIGHT_URL}))
        self.assertEqual(self._service._get_insight_url(insight=SAMPLE_INSIGHT), SAMPLE_INSIGHT_URL)

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=GET_INSIGHT_REPORT,
            operation='getReportUrl',
            variables=dict(insight_name=SAMPLE_INSIGHT, report_name=f'{SAMPLE_INSIGHT}.csv')
        )

    def test_get_insights_url_not_available(self):
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(data=Box({'url': None}))

        with self.assertRaises(click.exceptions.Abort):
            self._service._get_insight_url(insight=SAMPLE_INSIGHT)

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=GET_INSIGHT_REPORT,
            operation='getReportUrl',
            variables=dict(insight_name=SAMPLE_INSIGHT, report_name=f'{SAMPLE_INSIGHT}.csv')
        )

    @patch('montecarlodata.insights.data_insights.urllib')
    @patch('montecarlodata.insights.data_insights.mkdirs')
    def test_save_insight_to_disk(self, mkdir_mock, urllib_mock):
        destination = 'folder1/folder2/object'
        self._service._save_insight_to_disk(insight_url=SAMPLE_INSIGHT_URL, destination=destination)
        mkdir_mock.assert_called_once_with('folder1/folder2')
        urllib_mock.request.urlretrieve.assert_called_once_with(SAMPLE_INSIGHT_URL, destination)

    @patch('montecarlodata.insights.data_insights.urllib')
    @patch('montecarlodata.insights.data_insights.mkdirs')
    def test_save_insight_to_disk_with_no_destination(self, mkdir_mock, urllib_mock):
        with self.assertRaises(click.exceptions.Abort):
            self._service._save_insight_to_disk(insight_url=SAMPLE_INSIGHT_URL, destination=None)
        mkdir_mock.assert_not_called()
        urllib_mock.request.urlretrieve.assert_not_called()

    @patch('montecarlodata.insights.data_insights.AwsClientWrapper')
    @patch('montecarlodata.insights.data_insights.requests')
    def test_save_insight_to_s3(self, request_mock, utils_mock):
        destination = 'bucket/prefix/object'
        request_mock.get.return_value = MockRequest

        self._service._save_insight_to_s3(insight_url=SAMPLE_INSIGHT_URL, destination=destination)
        request_mock.get.assert_called_once_with(SAMPLE_INSIGHT_URL, stream=True)
        utils_mock().upload_stream_to_s3.assert_called_once_with(
            data=SAMPLE_RAW,
            bucket='bucket',
            key='prefix/object'
        )

    @patch('montecarlodata.insights.data_insights.AwsClientWrapper')
    @patch('montecarlodata.insights.data_insights.requests')
    def test_save_insight_to_s3_with_bad_destination(self, request_mock, utils_mock):
        with self.assertRaises(click.exceptions.Abort):
            self._service._save_insight_to_s3(insight_url=SAMPLE_INSIGHT_URL, destination='bucket')
        request_mock.assert_not_called()
        utils_mock.assert_not_called()

        request_mock.reset_mock()
        utils_mock.reset_mock()

        with self.assertRaises(click.exceptions.Abort):
            self._service._save_insight_to_s3(insight_url=SAMPLE_INSIGHT_URL, destination='bucket/')
        request_mock.assert_not_called()
        utils_mock.assert_not_called()



import configparser
from collections import OrderedDict
from unittest import TestCase
from unittest.mock import Mock, patch, mock_open, call

from montecarlodata.config import Config, ConfigManager

_SAMPLE_PROFILE_NAME = 'foo'
_SAMPLE_BASE_PATH = 'bar'
_SAMPLE_FILE = f'{_SAMPLE_BASE_PATH}/profiles.ini'
_SAMPLE_OPTIONS = OrderedDict({
    'mcd_id': '1234',
    'mcd_token': 'a-test-token-with-fifty-six-characters-for-verifications',
    'mcd_api_endpoint': 'https://api.getmontecarlo.com/graphql',
    'aws_profile': 'foo',
    'aws_region': 'home'
})


class ConfigTest(TestCase):
    def setUp(self) -> None:
        self._parser_mock = Mock()

        self._service = ConfigManager(
            profile_name=_SAMPLE_PROFILE_NAME,
            base_path=_SAMPLE_BASE_PATH,
            config_parser=self._parser_mock
        )

    def test_setup(self):
        self._parser_mock.read.assert_called_once_with(_SAMPLE_FILE)

    @patch('montecarlodata.config.mkdirs')
    @patch('builtins.open', new_callable=mock_open, read_data='data')
    def test_write(self, mock_file, create_directory_mock):
        self._parser_mock.sections.return_value = []
        self._service.write(**_SAMPLE_OPTIONS)

        self._parser_mock.add_section.assert_called_once_with(_SAMPLE_PROFILE_NAME)

        self._parser_mock.set.assert_has_calls([
            call(_SAMPLE_PROFILE_NAME, k, v) for k, v in _SAMPLE_OPTIONS.items()
        ])

        mock_file.assert_called_once_with(_SAMPLE_FILE, 'w')
        create_directory_mock.assert_called_once_with(_SAMPLE_BASE_PATH)

    @patch('montecarlodata.errors.echo_error')
    @patch('montecarlodata.config.mkdirs')
    @patch('builtins.open', new_callable=mock_open, read_data='data')
    def test_write_invalid_token(self, mock_file, create_directory_mock, mock_echo):
        self._parser_mock.sections.return_value = []
        INVALID_SHORT_TOKEN = 'invalid-short-token'

        self._service.write(**{**_SAMPLE_OPTIONS, 'mcd_token': INVALID_SHORT_TOKEN})
        mock_echo.assert_called_once()

        self._parser_mock.add_section.assert_called_once_with(_SAMPLE_PROFILE_NAME)
        self._parser_mock.set.called_once_with(_SAMPLE_PROFILE_NAME, 'mcd_token', INVALID_SHORT_TOKEN)

        mock_file.assert_not_called()
        create_directory_mock.assert_not_called()

    def test_read(self):
        self._parser_mock.get.side_effect = _SAMPLE_OPTIONS.values()
        self.assertEqual(self._service.read(), Config(**_SAMPLE_OPTIONS))

    def test_read_with_no_section(self):
        self._parser_mock.get.side_effect = configparser.NoSectionError(_SAMPLE_PROFILE_NAME)
        self.assertIsNone(self._service.read())



import json
import os
from typing import Optional, Dict

import click
from tabulate import tabulate

from montecarlodata.common.data import ConnectionOptions, ValidationResult, OnboardingConfiguration
from montecarlodata.common.user import UserService
from montecarlodata.config import Config
from montecarlodata.errors import complain_and_abort, prompt_connection
from montecarlodata.integrations.onboarding.fields import S3_CERT_MECHANISM, GQL_TO_FRIENDLY_CONNECTION_MAP, \
    DATA_LAKE_WAREHOUSE_TYPE, EXPECTED_ADD_CONNECTION_RESPONSE_FIELD, CONFIRM_CONNECTION_VERBIAGE, \
    VALIDATIONS_FAILED_VERBIAGE, \
    SKIP_ADD_CONNECTION_VERBIAGE, CONNECTION_TEST_SUCCESS_VERBIAGE, CONNECTION_TEST_FAILED_VERBIAGE, \
    ADD_CONNECTION_FAILED_VERBIAGE, ADD_CONNECTION_SUCCESS_VERBIAGE, MAIN_CONNECTION_TYPES
from montecarlodata.queries.onboarding import ADD_CONNECTION_MUTATION
from montecarlodata.utils import GqlWrapper, AwsClientWrapper


class BaseOnboardingService:
    _VALIDATION_STATUS_HEADERS = ['Validation', 'Result', 'Details']
    _VALIDATION_PASSED = 'Passed'
    _VALIDATION_FAILED = 'Failed'

    def __init__(self, config: Config, user_service: Optional[UserService] = None,
                 request_wrapper: Optional[GqlWrapper] = None,
                 aws_wrapper: Optional[AwsClientWrapper] = None):

        self._abort_on_error = True  # Aborts methods with deco on unhandled error
        self._dc_outputs = None

        self._user_service = user_service or UserService(config=config)
        self._request_wrapper = request_wrapper or GqlWrapper(config)
        self._aws_wrapper = aws_wrapper or AwsClientWrapper(profile_name=config.aws_profile,
                                                            region_name=config.aws_region)

    def onboard(self, **kwargs):
        """
        Convenience wrapper to validate and add a connection.
        """
        onboarding_config = OnboardingConfiguration(**kwargs)  # not passed as arg to avoid having to change all clients
        onboarding_config.connection_options = self._build_connection_options(**onboarding_config.connection_options)

        onboarding_config.warehouse_name = onboarding_config.connection_options.monolith_base_payload.pop(
            'warehouseName',
            onboarding_config.connection_type
        )
        # Prefer explicitly set types.
        if not onboarding_config.warehouse_type:
            onboarding_config.warehouse_type = onboarding_config.connection_options.monolith_base_payload.pop(
                'warehouseType',
                DATA_LAKE_WAREHOUSE_TYPE
            )

        result = self._validate_connection(
            query=onboarding_config.validation_query,
            response_field=onboarding_config.validation_response,
            **onboarding_config.connection_options.monolith_base_payload
        )

        if not onboarding_config.connection_options.validate_only:
            prompt = VALIDATIONS_FAILED_VERBIAGE if result.has_warnings else CONFIRM_CONNECTION_VERBIAGE
            prompt_connection(message=prompt, skip_prompt=onboarding_config.connection_options.auto_yes)
            self._add_connection(temp_path=result.credentials_key, onboarding_config=onboarding_config)
        else:
            click.echo(SKIP_ADD_CONNECTION_VERBIAGE)

    def handle_cert(self, cert_prefix: str, options: Dict) -> None:
        """
        Handles cert payload from either an s3 path or file. Uploading the latter
        Options is updated if successful.
        """
        if options.get('cert_file') is not None:
            bucket_name = self._get_dc_property(prop='PrivateS3BucketArn').split(':')[5]  # get name from arn
            object_name = os.path.join(cert_prefix, os.path.basename(options['cert_file']))
            self._aws_wrapper.upload_file(bucket_name=bucket_name, object_name=object_name,
                                          file_path=options['cert_file'])

            click.echo(f"Uploaded '{options['cert_file']}' to s3://{bucket_name}/{object_name}")
            options['cert_s3'] = object_name

        ssl_options = {}
        if options.get('cert_s3') is not None:
            # reformat to generic options and specify a mechanism
            ssl_options = {'mechanism': S3_CERT_MECHANISM, 'cert': options.pop('cert_s3')}
        options.pop('cert_file', None)

        if options.get('skip_cert_verification') is not None:
            ssl_options['skip_verification'] = options.pop('skip_cert_verification')

        if ssl_options:
            options['ssl_options'] = ssl_options

    def _validate_connection(self, query: str, response_field: str, **kwargs) -> ValidationResult:
        """
        Validate connection before adding using the expected gql response_field (e.g. testPrestoCredentials)
        """
        response = self._request_wrapper.make_request_v2(
            query=query,
            operation=response_field,
            variables=kwargs
        )

        summary = self._build_validation_summary(response.data)
        temp_path = response.data.get('key')
        warnings = response.data.get('warnings')

        if temp_path is not None:
            click.echo(f'{CONNECTION_TEST_SUCCESS_VERBIAGE}{summary}')
            return ValidationResult(
                has_warnings=warnings is not None and len(warnings) > 0,
                credentials_key=temp_path
            )

        complain_and_abort(f'{CONNECTION_TEST_FAILED_VERBIAGE}{summary}')

    def _build_validation_summary(self, response: Dict, table_format: Optional[str] = 'fancy_grid') -> str:
        data = []
        if response.get('validations'):
            for passed in response.get('validations'):
                data.append([
                    passed.get('message'),
                    self._VALIDATION_PASSED,
                    None
                ])

        if response.get('warnings'):
            for failed in response.get('warnings', []):
                data.append([
                    failed.get('message'),
                    self._VALIDATION_FAILED,
                    failed.get('data', {}).get('error')
                ])

        if data:
            return '\n' + tabulate(data, headers=self._VALIDATION_STATUS_HEADERS, tablefmt=table_format)

        # do not provide a summary if no detailed response is available
        return ''

    def _add_connection(self, temp_path: str, onboarding_config: OnboardingConfiguration) -> bool:
        """
        Add connection and setup any associated jobs
        """
        connection_request = {'key': temp_path, 'connectionType': onboarding_config.connection_type}
        connection_request.update(self._disambiguate_warehouses(onboarding_config))

        # Set optional properties
        if onboarding_config.warehouse_name:
            connection_request['name'] = onboarding_config.warehouse_name
        if onboarding_config.job_types:
            connection_request['jobTypes'] = onboarding_config.job_types
        if onboarding_config.job_limits:
            connection_request['jobLimits'] = json.dumps(onboarding_config.job_limits)
        if onboarding_config.connection_options and onboarding_config.connection_options.dc_id:
            connection_request['dcId'] = onboarding_config.connection_options.dc_id

        response = self._request_wrapper.make_request_v2(
            query=onboarding_config.connection_query or ADD_CONNECTION_MUTATION,
            operation=onboarding_config.connection_response or EXPECTED_ADD_CONNECTION_RESPONSE_FIELD,
            variables=connection_request,
        )

        connection_id = response.data.get('connection', {}).get('uuid')
        if connection_id is not None:
            click.echo(f"{ADD_CONNECTION_SUCCESS_VERBIAGE}"
                       f"{GQL_TO_FRIENDLY_CONNECTION_MAP.get(onboarding_config.connection_type, onboarding_config.connection_type.capitalize())}.")
            return True
        complain_and_abort(ADD_CONNECTION_FAILED_VERBIAGE)

    def _disambiguate_warehouses(self, onboarding_config: OnboardingConfiguration) -> Dict:
        """
        Determine type of connection request to build using the following criteria -
            1) If it is a main connection type or a traditional warehouse (i.e. not a lake), create a new warehouse.
            2) If a name has been passed, create the connection in that warehouse.
            3) If no name has been passed but there is only one warehouse, use it.
            4) Otherwise fail.
        """
        warehouse_type = onboarding_config.warehouse_type or DATA_LAKE_WAREHOUSE_TYPE  # default to data-lake
        connection_type = onboarding_config.connection_type
        create_warehouse = onboarding_config.create_warehouse

        # Main connections require the creation of a new warehouse.
        if (connection_type in MAIN_CONNECTION_TYPES or warehouse_type != DATA_LAKE_WAREHOUSE_TYPE) and create_warehouse:
            return {'createWarehouseType': warehouse_type}

        name = onboarding_config.warehouse_name

        dw_id = self._get_warehouse_id_from_name(name)
        return {'dwId': dw_id}

    def _get_warehouse_id_from_name(self, name: Optional[str]) -> str:
        warehouses = self._user_service.warehouses

        # If it is not a new warehouse, find the warehouse with the given name to create the new connection belonging
        # to it. If the name is not found, it is an error.
        if name:
            warehouses = [warehouse for warehouse in warehouses if warehouse.get('name') == name]
            if warehouses:
                return warehouses[0]['uuid']
            complain_and_abort('Could not find warehouse with given name')
        # Name is optional if there is only one warehouse.
        elif len(warehouses) == 1:
            return warehouses[0]['uuid']
        # If more than one warehouse exists, name is required.
        else:
            complain_and_abort('Name is required to disambiguate lake warehouses')

    def _get_dc_property(self, prop: str) -> Optional[str]:
        """
        Retrieve property from DC stack outputs
        """
        self._dc_outputs = self._dc_outputs or self._aws_wrapper.get_stack_outputs(
            self._user_service.active_collector['stackArn'])  # cache lookup
        for output in self._dc_outputs:
            if output['OutputKey'] == prop:
                return output['OutputValue']

    @staticmethod
    def _build_connection_options(**kwargs) -> ConnectionOptions:
        """
        Create connection options from arguments
        """
        connection_options = ConnectionOptions(**kwargs)

        if connection_options.monolith_connection_payload:
            connection_options.monolith_base_payload['connectionOptions'] = connection_options.monolith_connection_payload
        return connection_options



import click
import click_config_file

from montecarlodata import settings
from montecarlodata.iac.mc_config_service import MonteCarloConfigService
from montecarlodata.monitors.monitor_service import MonitorService


@click.group(help='Manage monitors.')
def monitors():
    """
    Group for any monitor related subcommands
    """
    pass

@monitors.command(help='Compile monitor configuration.')
@click.option('--project-dir', required=False, help='Base directory of MC project (where montecarlo.yml is located). By default, this is set to the current working directory')
@click.pass_obj
def compile(ctx, project_dir):
    MonteCarloConfigService(config=ctx['config'], project_dir=project_dir).compile()

@monitors.command(help='Compile and apply monitor configuration.')
@click.option('--project-dir', required=False, help='Base directory of MC project (where montecarlo.yml is located). By default, this is set to the current working directory')
@click.option('--namespace', required=True, help='Namespace of monitors configuration.')
@click.option('--dry-run', required=False, default=False, show_default=True, is_flag=True, help='Dry run (just shows planned changes but doesn\'t apply them.')
@click.pass_obj
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def apply(ctx, project_dir, namespace, dry_run):
    MonteCarloConfigService(config=ctx['config'], project_dir=project_dir).apply(namespace, dry_run=dry_run)

@monitors.command(help='Delete monitor configuration.')
@click.option('--project-dir', required=False, help='Base directory of MC project (where montecarlo.yml is located). By default, this is set to the current working directory')
@click.option('--namespace', required=True, help='Namespace of monitors configuration.')
@click.option('--dry-run', required=False, default=False, show_default=True, is_flag=True, help='Dry run (just shows planned changes but doesn\'t apply them.')
@click.pass_obj
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def delete(ctx, project_dir, namespace, dry_run):
    MonteCarloConfigService(config=ctx['config'], project_dir=project_dir).delete(namespace, dry_run=dry_run)

@monitors.command(name='list', help='List monitors ordered by update recency.')
@click.option('--limit', required=False, default=100, type=click.INT, show_default=True, help='Max number of monitors to list.')
@click.option(
    '--monitor-type',
    required=False,
    type=click.Choice(MonitorService.MONITOR_TYPES, case_sensitive=False),
    help='List monitors with monitor_type'
)
@click.option(
    '--namespace',
    required=False,
    help='List only monitors in this namespace'
)
@click.pass_obj
def list_monitors(ctx, monitor_type, limit, namespace):
    monitor_types = None
    namespaces = [namespace] if namespace else None
    if monitor_type:
        if monitor_type == MonitorService.PSEUDO_MONITOR_TYPE_CB_COMPATIBLE:
            monitor_type = MonitorService.MONITOR_TYPE_CUSTOM_SQL
        monitor_types = [monitor_type]
    MonitorService().list_monitors(limit, namespaces, monitor_types)

@monitors.command(name='namespaces', help='List all namespaces.')
@click.option('--limit', required=False, default=100, type=click.INT, show_default=True,
              help='Max number of namespaces to list.')
@click.pass_obj
def list_namespaces(ctx, limit):
    MonteCarloConfigService(config=ctx['config']).list_namespaces(limit)



from unittest import TestCase
from unittest.mock import Mock, patch, call

from box import Box
from pycarlo.core import Client, Mutation, Query
from parameterized import parameterized

from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.data_lake.databricks import DatabricksOnboardingService, \
    DEFAULT_GATEWAY_URL, DEFAULT_SECRET_NAME, DEFAULT_SECRET_SCOPE
from montecarlodata.queries.onboarding import TEST_DATABRICKS_CRED_MUTATION
from montecarlodata.utils import GqlWrapper, AwsClientWrapper
from tests.test_base_onboarding import _SAMPLE_BASE_OPTIONS
from tests.test_common_user import _SAMPLE_CONFIG
from montecarlodata.integrations.onboarding.fields import (
    DATABRICKS_METASTORE_CONNECTION_TYPE,
    DATABRICKS_DELTA_CONNECTION_TYPE,
    DATA_LAKE_WAREHOUSE_TYPE,
)


class DatabricksMetastoreOnboardingTest(TestCase):
    def setUp(self) -> None:
        self._mc_client_mock = Mock(autospec=Client)
        self._user_service_mock = Mock(autospec=UserService)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = DatabricksOnboardingService(
            _SAMPLE_CONFIG,
            mc_client=self._mc_client_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock
        )

    @parameterized.expand([
        (DATABRICKS_METASTORE_CONNECTION_TYPE, [], {}, 3),
        (DATABRICKS_DELTA_CONNECTION_TYPE, ['hive'], {'project_id': 'hive'}, 4),
        (DATABRICKS_DELTA_CONNECTION_TYPE, ['awsdatacatalog'], {'project_id': 'awsdatacatalog'}, 4),
    ])
    @patch.object(DatabricksOnboardingService, 'onboard')
    def test_databricks_flow(self, connection_type, projects, expected_projects, expected_call_count, onboard_mock):
        options = {
            'databricks_workspace_url': 'databricks_workspace_url',
            'databricks_workspace_id': 'databricks_workspace_id',
            'databricks_cluster_id': 'databricks_cluster_id',
            'databricks_token': 'databricks_token',
            'databricks_secret_key': DEFAULT_SECRET_NAME,
            'databricks_secret_scope': DEFAULT_SECRET_SCOPE,
        }
        job_info = dict(
            databricks_job_id='databricks_job_id',
            databricks_job_name='databricks_job_name',
            databricks_notebook_path='databricks_notebook_path',
            databricks_notebook_source='databricks_notebook_path',
            databricks_notebook_version='123',
        )

        self._mc_client_mock.side_effect = [
            Mock(),
            Mock(),
            Box(dict(
                create_databricks_notebook_job=dict(
                    databricks=dict(
                        workspace_job_id=job_info['databricks_job_id'],
                        workspace_job_name=job_info['databricks_job_name'],
                        workspace_notebook_path=job_info['databricks_notebook_path'],
                        notebook_source=job_info['databricks_notebook_source'],
                        notebook_version=job_info['databricks_notebook_version'],
                    ),
                )
            )),
            Box(dict(get_projects=dict(projects=projects)))
        ]

        self._service.onboard_databricks_metastore(connection_type=connection_type, **options, **_SAMPLE_BASE_OPTIONS)

        expected_options = {**options, **_SAMPLE_BASE_OPTIONS}
        expected_job_limits = {
            **job_info,
            'integration_gateway': {
                'gateway_url': DEFAULT_GATEWAY_URL,
                'databricks_secret_key': DEFAULT_SECRET_NAME,
                'databricks_secret_scope': DEFAULT_SECRET_SCOPE
            },
            **expected_projects,
        }

        onboard_mock.assert_called_once_with(
            validation_query=TEST_DATABRICKS_CRED_MUTATION,
            validation_response='testDatabricksCredentials',
            connection_type=connection_type,
            job_limits=expected_job_limits,
            **expected_options
        )
        self.assertEqual(self._mc_client_mock.call_count, expected_call_count)

    @patch.object(DatabricksOnboardingService, 'onboard')
    def test_databricks_gql_api_calls(self, onboard_mock):
        options = {
            'databricks_workspace_url': 'databricks_workspace_url',
            'databricks_workspace_id': 'databricks_workspace_id',
            'databricks_cluster_id': 'databricks_cluster_id',
            'databricks_token': 'databricks_token',
            'databricks_secret_key': DEFAULT_SECRET_NAME,
            'databricks_secret_scope': DEFAULT_SECRET_SCOPE,
            'dc_id': 'data collector id'
        }
        job_info = dict(
            databricks_job_id='databricks_job_id',
            databricks_job_name='databricks_job_name',
            databricks_notebook_path='databricks_notebook_path',
            databricks_notebook_source='databricks_notebook_path',
            databricks_notebook_version='123',
        )

        databricks_config = {
            'databricks_workspace_url': 'databricks_workspace_url',
            'databricks_workspace_id': 'databricks_workspace_id',
            'databricks_cluster_id': 'databricks_cluster_id',
            'databricks_token': 'databricks_token',
        }

        self._mc_client_mock.side_effect = [
            Mock(),
            Mock(),
            Box(dict(
                create_databricks_notebook_job=dict(
                    databricks=dict(
                        workspace_job_id=job_info['databricks_job_id'],
                        workspace_job_name=job_info['databricks_job_name'],
                        workspace_notebook_path=job_info['databricks_notebook_path'],
                        notebook_source=job_info['databricks_notebook_source'],
                        notebook_version=job_info['databricks_notebook_version'],
                    )
                )
            )),
            Box(dict(get_projects=dict(projects=[])))
        ]

        self._service.onboard_databricks_metastore(connection_type=DATABRICKS_METASTORE_CONNECTION_TYPE, **options, **_SAMPLE_BASE_OPTIONS)

        expected_validation_call =  Query()
        expected_validation_call.validate_connection_type(
            warehouse_type=DATA_LAKE_WAREHOUSE_TYPE,
            connection_type=DATABRICKS_METASTORE_CONNECTION_TYPE
        )

        expected_secret_call = Mutation()
        expected_secret_call.create_databricks_secret(
            databricks_config=databricks_config,
            secret_name=DEFAULT_SECRET_NAME,
            scope_name=DEFAULT_SECRET_SCOPE,
            connection_options={'dc_id': 'data collector id'},
        )

        expected_notebook_job_call = Mutation()
        expected_notebook_job_call.create_databricks_notebook_job(databricks_config=databricks_config, connection_options={'dc_id': 'data collector id'})

        expected_calls = [
            call(expected_validation_call),
            call(expected_secret_call),
            call(expected_notebook_job_call)
        ]
        self.assertEqual(len(self._mc_client_mock.call_args_list), len(expected_calls))

        for i, expected_call in enumerate(expected_calls):
            # This is the same way that the SDK tests that two calls are the same 
            # https://github.com/monte-carlo-data/python-sdk/blob/62f1ab7a8404119da4621b224e77bbbdd5dc938c/tests/test_operations.py#L21
            self.assertEqual(str(self._mc_client_mock.call_args_list[i]).strip(), str(expected_call).strip())

    def test_update_databricks_notebook_gql_calls(self):
        options = {
            'connection_id': '123456789'
        }
        self._service.update_databricks_notebook(**options)
        expected_mutation = Mutation()
        expected_mutation.update_databricks_notebook(**options)
        expected_call = call(expected_mutation)
        self.assertEqual(str(self._mc_client_mock.call_args_list[0]).strip(), str(expected_call).strip())

    def test_get_databricks_job_info_gql_calls(self):
        options = {
            'connection_id': '123456789'
        }
        self._mc_client_mock.return_value = Mock(
            get_databricks_metadata_job_info = [
                Mock(
                    workspace_job_id='123456',
                    workspace_job_name='test_job_name',
                    workspace_notebook_path='/some/path',
                    notebook_source='test_source',
                    notebook_version='123',
                )
            ]
        )
        self._service.get_databricks_job_info(**options)
        expected_query = Query()
        expected_query.get_databricks_metadata_job_info(**options)
        expected_call = call(expected_query)
        self.assertEqual(str(self._mc_client_mock.call_args_list[0]).strip(), str(expected_call).strip())

    def test_get_current_databricks_notebook_version_gql_calls(self):
        self._service.get_current_databricks_notebook_version()
        expected_query = Query()
        expected_query.get_current_databricks_notebook_version()
        expected_call = call(expected_query)
        self.assertEqual(str(self._mc_client_mock.call_args_list[0]).strip(), str(expected_call).strip())




from unittest import TestCase
from unittest.mock import Mock, patch

from montecarlodata.common.data import ValidationResult
from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.etl.dbt_cloud import DbtCloudOnboardingService
from montecarlodata.integrations.onboarding.fields import EXPECTED_DBT_CLOUD_RESPONSE_FIELD, \
    EXPECTED_ADD_CONNECTION_RESPONSE_FIELD
from montecarlodata.queries.onboarding import TEST_DBT_CLOUD_CRED_MUTATION, ADD_CONNECTION_MUTATION
from montecarlodata.utils import GqlWrapper, AwsClientWrapper
from tests.test_common_user import _SAMPLE_CONFIG

_SAMPLE_WAREHOUSE = {
    'uuid': 'my-snowflake-warehouse-uuid',
    'name': 'snowflake',
    'connectionType': 'SNOWFLAKE',
    'connections': [
        {
            'uuid': 'my-snowflake-connection-uuid',
            'type': 'SNOWFLAKE',
            'createdOn': '2020-03-28T23:51:04.775647+00:00',
            'jobTypes': ['metadata', 'query_logs', 'sql_query', 'json_schema'],
            'connectionIdentifier': {
                'key': 'account',
                'value': 'hda34492.us-east-1'
            }
        }]
}


class DbtCloudOnboardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = DbtCloudOnboardingService(
            _SAMPLE_CONFIG,
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock
        )
        self._service._disable_handle_errors = True

    @patch.object(DbtCloudOnboardingService, '_validate_connection')
    @patch('montecarlodata.integrations.onboarding.base.prompt_connection')
    def test_onboard_dbt_cloud(self, prompt_mock, validation_mock):
        self._user_service_mock.warehouses = [
            _SAMPLE_WAREHOUSE
        ]
        validation_mock.return_value = ValidationResult(has_warnings=False, credentials_key='path')

        self._service.onboard_dbt_cloud(warehouseName=_SAMPLE_WAREHOUSE['name'])

        validation_mock.assert_called_once_with(
            query=TEST_DBT_CLOUD_CRED_MUTATION,
            response_field=EXPECTED_DBT_CLOUD_RESPONSE_FIELD
        )

        self._request_wrapper_mock.make_request_v2.assert_called_with(
            operation=EXPECTED_ADD_CONNECTION_RESPONSE_FIELD,
            query=ADD_CONNECTION_MUTATION,
            variables={
                'key': 'path',
                'connectionType': 'dbt-cloud-v2',
                'name': _SAMPLE_WAREHOUSE['name'],
                'dwId': 'my-snowflake-warehouse-uuid'  # check that dwId is passed as a parameter
            })



import os
import pathlib
from typing import Callable, Dict
from unittest import TestCase
from unittest.mock import Mock

import click
from box import Box

from montecarlodata.common.common import read_as_json_string
from montecarlodata.common.data import MonolithResponse
from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.data_lake.events import EventsOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_CONFIGURE_AIRFLOW_LOG_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_CONFIGURE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_CONFIGURE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_DISABLE_AIRFLOW_LOG_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_DISABLE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_DISABLE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD
)
from montecarlodata.queries.onboarding import (
    CONFIGURE_AIRFLOW_LOG_EVENTS_MUTATION,
    CONFIGURE_METADATA_EVENTS_MUTATION,
    CONFIGURE_QUERY_LOG_EVENTS_MUTATION,
    DISABLE_AIRFLOW_LOG_EVENTS_MUTATION,
    DISABLE_METADATA_EVENTS_MUTATION,
    DISABLE_QUERY_LOG_EVENTS_MUTATION
)
from montecarlodata.utils import GqlWrapper, AwsClientWrapper
from tests.test_common_user import _SAMPLE_CONFIG


class EventsOnboardingTest(TestCase):

    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = EventsOnboardingService(
            _SAMPLE_CONFIG,
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock
        )

    def test_configure_airflow_events(self):
        # given
        kwargs = {
            'name': 'warehouse'
        }

        self._test(
            func=self._service.configure_airflow_events,
            kwargs=kwargs,
            expected_mutation=CONFIGURE_AIRFLOW_LOG_EVENTS_MUTATION,
            expected_operation=EXPECTED_CONFIGURE_AIRFLOW_LOG_EVENTS_GQL_RESPONSE_FIELD,
            expected_variables=kwargs
        )

    def test_configure_metadata_events(self):
        kwargs = {
            'connection_type': 'glue',
            'name': 'warehouse'
        }

        self._test(
            func=self._service.configure_metadata_events,
            kwargs=kwargs,
            expected_mutation=CONFIGURE_METADATA_EVENTS_MUTATION,
            expected_operation=EXPECTED_CONFIGURE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
            expected_variables=kwargs
        )

    def test_configure_query_log_events(self):
        mapping_file = os.path.join(pathlib.Path(__file__).parent.resolve(), 'sample_mapping.json')
        kwargs = {
            'connection_type': 'hive-s3',
            'format_type': 'custom',
            'location': 'foo',
            'source_format': 'bar',
            'mapping_file': mapping_file,
            'name': 'warehouse'
        }

        expected_variables = {
            **kwargs,
            'mapping': read_as_json_string(mapping_file)
        }
        expected_variables.pop('mapping_file')

        self._test(
            func=self._service.configure_query_log_events,
            kwargs=kwargs,
            expected_mutation=CONFIGURE_QUERY_LOG_EVENTS_MUTATION,
            expected_operation=EXPECTED_CONFIGURE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD,
            expected_variables=expected_variables
        )

    def test_disable_airflow_log_events(self):
        kwargs = {
            'name': 'warehouse'
        }

        self._test(
            func=self._service.disable_airflow_log_events,
            kwargs=kwargs,
            expected_mutation=DISABLE_AIRFLOW_LOG_EVENTS_MUTATION,
            expected_operation=EXPECTED_DISABLE_AIRFLOW_LOG_EVENTS_GQL_RESPONSE_FIELD,
            expected_variables=kwargs
        )

    def test_disable_metadata_events(self):
        kwargs = {
            'name': 'warehouse'
        }

        self._test(
            func=self._service.disable_metadata_events,
            kwargs=kwargs,
            expected_mutation=DISABLE_METADATA_EVENTS_MUTATION,
            expected_operation=EXPECTED_DISABLE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
            expected_variables=kwargs
        )

    def test_disable_query_log_events(self):
        kwargs = {
            'name': 'warehouse'
        }

        self._test(
            func=self._service.disable_query_log_events,
            kwargs=kwargs,
            expected_mutation=DISABLE_QUERY_LOG_EVENTS_MUTATION,
            expected_operation=EXPECTED_DISABLE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD,
            expected_variables=kwargs
        )

    def _test(self,
              func: Callable,
              kwargs: Dict,
              expected_mutation: str,
              expected_operation: str,
              expected_variables: Dict):

        # test a successful GraphQL response
        self._verify_graphql_request(
            func=func,
            kwargs=kwargs,
            expected_mutation=expected_mutation,
            expected_operation=expected_operation,
            expected_variables=expected_variables,
            success=True
        )

        # test a failed GraphQL response
        self._verify_graphql_request(
            func=func,
            kwargs=kwargs,
            expected_mutation=expected_mutation,
            expected_operation=expected_operation,
            expected_variables=expected_variables,
            success=False
        )

    def _verify_graphql_request(self,
                                func: Callable,
                                kwargs: Dict,
                                success: bool,
                                expected_mutation: str,
                                expected_operation: str,
                                expected_variables: Dict):

        # given
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(data=Box({'success': success}))

        # when
        if success:
            func(**kwargs)
        else:
            self.assertRaises(click.exceptions.Abort, func, **kwargs)

        # then
        self._request_wrapper_mock.make_request_v2.assert_called_with(
            expected_mutation,
            expected_operation,
            expected_variables
        )
